#!/usr/bin/env python
# -*- coding: utf-8 -*

# LAMMPS driver for Langevin molecular dynamics
# Adapted from LAMMPS python wraper

import ctypes
import sys

import numpy as np

from sclmd.tools import get_atomname

args = "-screen none"


def GetDriver():
    from lammps import lammps

    class lammpsdriver(lammps):
        # create instance of LAMMPS
        def __init__(self, infile, label="",
                     constraints=[], tdir="./", lunit="Ang", eunit="eV", md2ang=0.06466,
                     name="", cmdargs=args.split(), ptr=None, comm=None
                     ):
            lammps.__init__(self, name=name, cmdargs=cmdargs,
                            ptr=None, comm=None)
            self.infile = infile
            self.md2ang = md2ang
            self.constraints = constraints
            self.label = label
            self.lunit = lunit
            self.eunit = eunit
            if self.eunit == "eV":
                self.para = 1.0
            elif self.eunit == "Kcal/mole":
                self.para = 0.043
            else:
                print("Wrong vaule in eunit")
                sys.exit(0)
            # start lammps
            self.start()

        def start(self):
            print("LAMMPS launched")
            # todo:better to set the unit to metals here again
            #self.command("units metal")

            #lines = open(self.infile, 'r').readlines()
            #for line in lines: self.command(line)
            self.commands_list(self.infile)
            self.type = np.array(self.gather_atoms("type", 0, 1))
            #self.mass = np.array(self.gather_atoms("mass",1,1))
            self.mass = self.extract_atom("mass", 2)
            self.number = self.get_natoms()
            self.els = []
            for type in self.type:
                self.els.append(self.mass[type])
            self.xyz = self.gather_atoms("x", 1, 3)
            self.conv = self.md2ang*np.array([3*[1.0/np.sqrt(mass)]
                                              for mass in self.els]).flatten()
            self.type = np.array(self.gather_atoms("type", 0, 1))
            #self.mass = np.array(self.gather_atoms("mass",1,1))
            self.axyz = []
            for i, a in enumerate(self.els):
                self.axyz.append([get_atomname(a), self.xyz[i*3],
                                  self.xyz[i*3+1], self.xyz[i*3+2]])
            # print(self.conv.shape)
            self.initforce()

        def quit(self):
            self.close()
            print("Mission completed.")

        def newx(self, q):
            newxyz = self.xyz + self.conv*q
            return newxyz.ctypes

        def absforce(self, q):
            self.scatter_atoms("x", 1, 3, self.newx(q))
            self.command("run 0")
            return self.para*self.conv*np.array(self.gather_atoms("f", 1, 3))

        def initforce(self):
            print("Calculate zero displacement force")
            self.f0 = self.absforce(np.zeros(3*self.number))

        def force(self, q):
            return self.absforce(q) - self.f0

        def energy(self, eargs="pe"):  # energy,eargs:"pe","ke" or "etotal".
            return self.get_thermo(eargs)

        def dynmat(self, q=None):
            if q is not None:
                self.scatter_atoms("x", 1, 3, self.newx(q))
                print("Calculate dynamical matrix")
            else:
                print("Calculate zero displacement dynamical matrix")
            self.command("dynamical_matrix all eskm 0.000001 file dynmat.dat")
            # Dynmat units in ps^-2, THz^2
            dynmatdat = np.loadtxt("dynmat.dat")
            dynlen = int(3*np.sqrt(len(dynmatdat)/3))
            rpc = 6.582119569e-4  # Reduced Planck constant in eV*ps
            # Dynmat units in eV^2
            return dynmatdat.reshape((dynlen, dynlen))*rpc**2

        # def updatedynmat(self, q):
        #    displacement = np.amax(np.abs(self.conv*q))
        #    if displacement > 2:
        #        print("Large displacement: %f ang, re-calcuate dynamical matrix" % displacement)
        #        return self.dynmat(q)
    return lammpsdriver



#!/usr/bin/env python
# -*- coding: utf-8 -*

import sys

import numpy as np

from sclmd.functions import antisymmetrize, chkShape, flinterp, mdot, symmetrize
from sclmd.noise import enoise, mf, phnoise


def exlist(a, indices):
    return a[indices]
    # return np.array([a[i] for i in indices])

# fourier transform of gamma (calculated directly, no fft)


def gamt(tl, wl, gwl, gam, eta_ad=0):
    '''
    calculate gam(t)

    tl      equal-spaced time points starting from 0 
    wl      equal-spaced omega points starting from 0
    gam     corresponding friction in omega space


    return the friction kernel in time domain
    '''
    gt = []
    # print "TG test was here!"
    # ea=0.01
    # print "Weird: ", wl[-1]
    # print wl[1]-wl[0],wl[2]-wl[1]
    if eta_ad == 0:
        print("eta=0")
        for t in tl:
            tm = []
            for i in range(len(wl)):
                tm.append(np.array(flinterp(wl[i], gwl, gam))*np.cos(wl[i]*t))
            # 2.0 account for the negative frequency part
            gt.append(2.0*np.mean(np.array(tm), axis=0)*wl[-1]/np.pi)
    else:
        print("eta!=0")
        for t in tl:
            tm = []
            for i in range(len(wl)):
                tm.append(np.array(flinterp(wl[i], gwl, gam))*wl[i]/(wl[i]-1j*eta_ad)*np.exp(-1j*wl[i]*t-eta_ad*t)+np.array(
                    flinterp(wl[i], gwl, gam))*wl[i]/(wl[i]+1j*eta_ad)*np.exp(+1j*wl[i]*t-eta_ad*t))
            gt.append(np.mean(np.array(tm), axis=0)*wl[-1]/np.pi)
            # print "TG imag:", max(np.imag(np.mean(np.array(tm[-1]),axis=0)*wl[-1]/np.pi))
    return np.array(np.real(gt))


class ebath:
    '''
    cats    center degree's of freedom connecting to electrons
    T       equlibrium temperature of electrons
    bias    applied bias
    wmax    cutoff energy of electron bath
    nw      number of energy points
    efric   friction matrix
    exim,exip
    zeta1,zeta2

    dt      md time step
    nmd     md steps

    noise   noise series
    kernel  friction kernel 
    ml      length of kernel(=1)

    ebath   include ebath or not
    '''

    def __init__(self, cats, T, dt, nmd, wmax=None, nw=None, bias=0.,
                 efric=None, exim=None, exip=None, zeta1=None, zeta2=None, classical=False, zpmotion=True):
        self.cats = np.array(cats, dtype='int')
        #self.cids = np.array([[3*c+0,3*c+1,3*c+2] for c in cats]).flatten()
        self.cids = np.array(cats, dtype='int')
        self.nc = len(self.cids)
        self.T, self.wmax = T, wmax
        self.nw, self.bias = nw, bias
        self.dt, self.nmd = dt, nmd
        self.cur = np.zeros(nmd)
        self.classical = classical
        self.zpmotion = zpmotion

        if nw is None or wmax is None:
            self.wl = None
        else:
            self.wl = [self.wmax*i/nw for i in range(nw)]

        self.CheckEmat(efric, exim, exip, zeta1, zeta2)

        # local friction
        self.ml = 1
        self.noise = None

    def CheckEmat(self, efric=None, exim=None, exip=None, zeta1=None, zeta2=None):
        '''
        check the matrix, and set the following variables:
            efric
            exim
            exip
            zeta1
            zeta2
            ebath
        '''
        # friction
        if efric is not None:
            print("ebath.CheckEmat: got efric, checking")
            n = chkShape(efric)
            if n != self.nc:
                print("ebath.CheckEmat: efric shape error!")
                sys.exit()
            print("ebath.setEmat: symmetrizing efric")
            self.efric = symmetrize(efric)
            self.kernel = np.array([self.efric])
            self.exip = np.zeros(shape=(n, n))
            self.exim = np.zeros(shape=(n, n))
            self.zeta1 = np.zeros(shape=(n, n))
            self.zeta2 = np.zeros(shape=(n, n))
            self.ebath = True
        else:
            print("ebath.CheckEmat: no efric provided, setting ebath to False")
            self.efric = None
            self.kernel = None
            self.exim = None
            self.exip = None
            self.zeta1 = None
            self.zeta2 = None
            self.ebath = False
            return

        # nc
        if exim is not None:
            print("ebath.CheckEmat: got exim, checking")
            n = chkShape(exim)
            if self.nc != n:
                print("ebath.CheckEmat: the dimension of exim is wrong!")
                sys.exit(0)
            print("ebath.setEmat: antisymmetrizing exim")
            self.exim = antisymmetrize(exim)

        if exip is not None:
            print("ebath.CheckEmat: got exip, checking")
            n = chkShape(exip)
            if self.nc != n:
                print("ebath.setEmat: the dimension of exip is wrong!")
                sys.exit(0)
            print("ebath.setEmat: symmetrizing exip")
            self.exip = symmetrize(exip)

        # renormalization
        if zeta1 is not None:
            print("ebath.CheckEmat: got zeta1, checking")
            n = chkShape(zeta1)
            if self.nc != n:
                print("ebath.setEmat: the dimension of zeta1 is wrong!")
                sys.exit(0)
            print("ebath.setEmat: symmetrizing zeta1")
            self.zeta1 = symmetrize(zeta1)

        # berry
        if zeta2 is not None:
            print("ebath.CheckEmat: got zeta2, checking")
            n = chkShape(zeta2)
            if self.nc != n:
                print("ebath.setEmat: the dimension of zeta2 is wrong!")
                sys.exit(0)
            print("ebath.setEmat: antisymmetrizing zeta2")
            self.zeta2 = antisymmetrize(zeta2)
        return

    def gnoi(self):
        '''
        electronic noise 
        '''
        if self.nmd is None:
            print("ebath.gnoi: nmd not set!")
            sys.exit()
        if self.dt is None:
            print("ebath.gnoi: dt not set!")
            sys.exit()
        if self.ebath is False:
            print("ebath.gnoi: ebath is False!")
            sys.exit()
        print("ebath.gnoi:classical: %r" % self.classical)
        print("ebath.gnoi:including zero point motion: %r" % self.zpmotion)
        self.noise = np.real(enoise(self.efric, self.exim, self.exip,
                                    self.bias, self.T, self.wmax, self.dt, self.nmd, self.classical, self.zpmotion))

    def GetSig(self):
        '''
        effective retarded self-energy in the wideband limit
        '''
        if self.wl is None:
            print("ebath.GetSig:wl is not set")
            sys.exit()
        else:
            wl = self.wl
        nw = len(wl)
        nc = chkShape(self.efric)
        self.sig = np.zeros((nw, nc, nc), np.complex)

        for i in range(nw):
            self.sig[i] = -1.j*wl[i]*(self.efric+self.bias*self.zeta2)\
                + self.bias*self.zeta1-self.bias*self.exim

    def SetMDsteps(self, dt, nmd):
        self.dt, self.nmd = dt, nmd
        print("ebath.SetMDsteps: memory len reset, you need to \
                regenerate the noise")

    def setbias(self, bias=0.0):
        '''
        set bias applied to the system
        '''
        self.bias = bias
        print("ebath.setbias: bias set to:%f" % self.bias)
        print("ebath.setbias: WARNING--BIAS CHANGED! YOU NEED TO REGENERATE THE NOISE!")

    def bforce(self, t, phis, qhis):
        '''
        return the force from baths, including noise and friction

        note that some of them are bias dependent

        BIAS IS DEFINED AS MUL-MUR
        '''
        f = self.noise[t % self.nmd]  # noise
        if not (self.exim.any() and self.zeta1.any() and self.zeta2.any()):
            for i in range(self.ml):  # friction,nc,rn,berry
                if self.ml == 1:
                    f = f-mdot(self.kernel[i], exlist(phis[i], self.cids))
                else:
                    print("WARNING: nonlocal electronic force not implemented!")
                    # stophere
                    f = f-mdot(self.kernel[i],
                               exlist(phis[i], self.cids))*self.dt
        else:
            for i in range(self.ml):  # friction,nc,rn,berry
                if self.ml == 1:
                    f = f-mdot(self.kernel[i], exlist(phis[i], self.cids))\
                        + mdot(self.bias*self.exim, exlist(qhis[0], self.cids))\
                        - mdot(self.bias*self.zeta1, exlist(qhis[0], self.cids))\
                        - mdot(self.bias*self.zeta2,
                               exlist(phis[0], self.cids))
                else:
                    print("WARNING: nonlocal electronic force not implemented!")
                    # stophere
                    f = f-mdot(self.kernel[i],
                               exlist(phis[i], self.cids))*self.dt
        return mf(f, self.cids, len(phis[0]))


class phbath:
    '''
    class for the phonon baths

    ##########################################################
    INPUT
    ##########################################################
    T           temperature of the bath
    cats        central system atoms connecting to the bath (python index)
    debye       Debye frequency of the lead material
    nw          number of sampling points in positive w space
    dt,nmd      time step and number of steps of md simulation
    ml          length of memory kernel
    mcof        the maximum frequency in wl is mcof*debye

    sig         self-energy matrix
    gamma       friction kernel in w space
    gwl         energy points of self-energy matrix

    K00,K01     the dynamical matrix of the semi-infinite leads
    V01         interaction matrix of lead with central system
    ##########################################################


    ##########################################################
    GENERATED VARIABLES
    ##########################################################
    nc          how many degrees of freedom are connected to central system
    local       if the friction is time-local or not
    wmax        cutoff frequency
    wl          list of frequencies, starting from 0
    cur         save the energy current to the center at each md time step
    noise       noise series
    kernel      friction kernel with length ml
    '''

    def __init__(self, T, cats, debye, nw, dt, nmd, ml=None, mcof=2.0, sig=None, gamma=None, gwl=None, K00=None, K01=None, V01=None, eta_ad=0, classical=False, zpmotion=True):
        self.classical = classical
        self.zpmotion = zpmotion
        self.T, self.debye, self.cats = T, debye, np.array(cats, dtype='int')
        #self.K00,self.K01,self.V01 = np.array(K00),np.array(K01),np.array(V01)
        self.K00, self.K01, self.V01 = K00, K01, V01
        self.dt, self.nmd, self.ml = dt, nmd, ml
        self.kernel = None
        #self.cids = np.array([[3*c+0, 3*c+1, 3*c+2] for c in cats]).flatten()
        self.cids = np.array(cats, dtype='int')
        self.nc = len(self.cids)
        self.wmax = mcof*debye
        self.local = False
        self.nw = nw
        self.wl = [self.wmax*i/nw for i in range(nw)]
        self.gamma = gamma
        self.sig = sig
        self.gwl = gwl
        self.cur = np.zeros(nmd)
        self.eta_ad = eta_ad
        #
        # initialise self.gamma
        if self.UseK():
            # calculate the real self-energy
            self.local = False
            print("phbath: Calculating self-energy is not implemented yet.")
            sys.exit(0)

        elif self.UseG() or self.UsePi():
            if self.UsePi():
                if len(self.sig[0]) != self.nc:
                    print("phbath: inconsist between cids and sig!")
                    sys.exit()
                self.ggamma()
            if self.UseG():
                if len(self.gamma[0]) != self.nc:
                    print("phbath: inconsist between cids and gamma!")
                    sys.exit()
        else:
            # use Debye model
            # friction coefficient from Debye frequency
            # see, Adelman&Doll, JCP, Vol.64,2375 (1976)
            phfric = debye*np.pi/6.0
            self.gamma = np.array([np.diag(phfric+np.zeros(int(self.nc)))])
            self.gwl = np.array([0])
            self.local = True
            self.ml = 1

    def SetMDsteps(self, dt, nmd):
        self.dt, self.nmd = dt, nmd
        print("phbath.SetMDsteps: memory len reset, you need to \
                regenerate the memory kernel and noise")

    def SetMemlen(self, len):
        self.ml = len
        print("phbath.SetMemlen: memory len reset, you need to \
                regenerate the memory kernel")

    def SetT(self, T):
        print("phbath.SetT: Set temperature  to %s\n" % T)
        self.T = T

    def UseG(self):
        if self.gamma is not None and self.gwl is not None:
            return True
        else:
            return False

    def UsePi(self):
        if self.sig is not None and self.gwl is not None:
            return True
        else:
            return False

    def UseK(self):
        if self.K00 is not None and self.K01 is not None \
           and self.V01 is not None:
            return True
        else:
            return False

    def ggamma(self):
        '''
        calculate gamma from self-energy
        Gamma(w) = -Im(Sig(w))/w
        '''
        if self.sig is not None:
            print("2TG test was here2!")
            Sig = self.sig
            wl = self.gwl
            a = []
            for i in range(len(wl)):
                if wl[i] == 0:
                    # a.append(-np.imag(Sig[0]))#
                    a.append(-np.imag(Sig[i+1])/wl[i+1])
                else:
                    # a.append(-np.imag(Sig[0]))#
                    a.append(-np.imag(Sig[i])/wl[i])
            self.gamma = np.array(a)
        else:
            print("phbath.Gamma: self.sig is not set, need it to calculate gamma")
            sys.exit()

    def gnoi(self):
        '''
        generate phonon noise using the friction kernel
        '''
        if self.dt is None or self.nmd is None:
            print("phbath.gnoi: the md information dt and nmd are not set!")
            sys.exit()
        print("regenerating noise")
        print("WARNING: remember to reset t=0 to use the new noise!")
        print("phbath.gnoi:classical: %r" % self.classical)
        print("phbath.gnoi:including zero point motion:%r" % self.zpmotion)
        self.noise = np.real(phnoise(self.gamma, self.gwl, self.T,
                                     self.wmax, self.dt, self.nmd, self.classical, self.zpmotion))
        # np.linspace(self.gwl[0],self.gwl[-1])

    def gmem(self):
        '''
        generate the memory kernel in time domain
        '''
        if self.ml is None or self.dt is None:
            print("phbath.gmem: length of memory kernel not set!")
            sys.exit()
        if self.local:
            print("phbath.gmem: local friction kernel is used")
            self.ml = 1
            self.kernel = self.gamma
        else:
            print("phbath.gmem: global friction kernel is used")
            tl = [self.dt*i for i in range(self.ml)]
            self.kernel = np.real(
                gamt(tl, self.wl, self.gwl, self.gamma, self.eta_ad))
            # update gamma to include artificial damping:
            if self.eta_ad != 0:
                # print "TG test FFT was here"
                #gamnew = np.zeros(self.kernel.shape)
                #fti = myfft(self.dt,self.kernel.shape[0])
                # for i in range(self.kernel.shape[1]):
                #    for j in range(self.kernel.shape[2]):
                #        gamnew[:,i,j]=(fti.Fourier1D(self.kernel[:,i,j])) #t->w, but only positive t!
                # self.gamma=np.array(2*np.real(gamnew)) # Real since its really a cos-transform (kernel(-t)=kernel(t)).
                gamnew = np.zeros(self.gamma.shape)
                for i in range(len(self.gwl)):
                    for it in range(self.kernel.shape[0]):
                        gamnew[i, :, :] = gamnew[i, :, :]+self.dt * \
                            self.kernel[it, :, :]*np.cos(self.gwl[i]*tl[it])
                # self.gamma=np.array(len(self.gwl)/self.kernel.shape[0]*np.real(gamnew)) # Real since its really a cos-transform (kernel(-t)=kernel(t)).
                self.gammaOld = self.gamma
                # Real since its really a cos-transform (kernel(-t)=kernel(t)).2*self.gwl[i]*
                self.gamma = np.array(np.real(gamnew))
                # print "Test igen:",self.gamma.shape

    def bforce(self, t, phis, qhis):
        '''
        return the force from baths, including noise and friction
        '''
        f = self.noise[t % self.nmd]  # noise
        for i in range(self.ml):  # friction
            if self.ml == 1:
                f = f-mdot(self.kernel[i], exlist(phis[i], self.cids))
            else:
                f = f-mdot(self.kernel[i], exlist(phis[i], self.cids))*self.dt
        return mf(f, self.cids, len(phis[0]))



#!/usr/bin/env python
# -*- coding: utf-8 -*

# LAMMPS driver for Langevin molecular dynamics
# Adapted from LAMMPS python wraper

import ctypes
import sys

import numpy as np
from lammps import lammps

from sclmd.tools import get_atomname

args = "-screen none"


class lammpsdriver(lammps):
    # create instance of LAMMPS
    def __init__(self, infile, label="",
                 constraints=[], tdir="./", lunit="Ang", eunit="eV", md2ang=0.06466,
                 name="", cmdargs=args.split(), ptr=None, comm=None
                 ):
        lammps.__init__(self, name=name, cmdargs=cmdargs, ptr=None, comm=None)
        self.infile = infile
        self.md2ang = md2ang
        self.constraints = constraints
        self.label = label
        self.lunit = lunit
        self.eunit = eunit
        if self.eunit == "eV":
            self.para = 1.0
        elif self.eunit == "Kcal/mole":
            self.para = 0.043
        else:
            print("Wrong vaule in eunit")
            sys.exit(0)
        # start lammps
        self.start()

    def start(self):
        print("LAMMPS launched")
        # todo:better to set the unit to metals here again
        #self.command("units metal")

        #lines = open(self.infile, 'r').readlines()
        #for line in lines: self.command(line)
        self.commands_list(self.infile)
        self.type = np.array(self.gather_atoms("type", 0, 1))
        #self.mass = np.array(self.gather_atoms("mass",1,1))
        self.mass = self.extract_atom("mass", 2)
        self.number = self.get_natoms()
        self.els = []
        for type in self.type:
            self.els.append(self.mass[type])
        self.xyz = self.gather_atoms("x", 1, 3)
        self.conv = self.md2ang*np.array([3*[1.0/np.sqrt(mass)]
                                          for mass in self.els]).flatten()
        self.type = np.array(self.gather_atoms("type", 0, 1))
        #self.mass = np.array(self.gather_atoms("mass",1,1))
        self.axyz = []
        for i, a in enumerate(self.els):
            self.axyz.append([get_atomname(a), self.xyz[i*3],
                              self.xyz[i*3+1], self.xyz[i*3+2]])
        # print(self.conv.shape)
        self.initforce()

    def quit(self):
        self.close()
        print("Mission completed.")

    def newx(self, q):
        newxyz = self.xyz + self.conv*q
        return newxyz.ctypes

    def absforce(self, q):
        self.scatter_atoms("x", 1, 3, self.newx(q))
        self.command("run 0")
        return self.para*self.conv*np.array(self.gather_atoms("f", 1, 3))

    def initforce(self):
        print("Calculate zero displacement force")
        self.f0 = self.absforce(np.zeros(3*self.number))

    def force(self, q):
        return self.absforce(q) - self.f0

    def energy(self, eargs="pe"):  # energy,eargs:"pe","ke" or "etotal".
        return self.get_thermo(eargs)

    def dynmat(self, q=None):
        if q is not None:
            self.scatter_atoms("x", 1, 3, self.newx(q))
            print("Calculate dynamical matrix")
        else:
            print("Calculate zero displacement dynamical matrix")
        self.command("dynamical_matrix all eskm 0.000001 file dynmat.dat")
        dynmatdat = np.loadtxt("dynmat.dat") # Dynmat units in ps^-2, THz^2
        dynlen = int(3*np.sqrt(len(dynmatdat)/3))
        rpc = 6.582119569e-4 # Reduced Planck constant in eV*ps
        return dynmatdat.reshape((dynlen, dynlen))*rpc**2 # Dynmat units in eV^2

    #def updatedynmat(self, q):
    #    displacement = np.amax(np.abs(self.conv*q))
    #    if displacement > 2:
    #        print("Large displacement: %f ang, re-calcuate dynamical matrix" % displacement)
    #        return self.dynmat(q)



#!/usr/bin/env python
# -*- coding: utf-8 -*

import sys

import numpy as np

import sclmd.units as U


class myfft:
    def __init__(self, dt, n):
        self.dt = dt
        self.N = n
        self.dw = 2*np.pi/dt/n

    def Fourier1D(self, a):
        """ 
        1D Fourier transform from t to w using our definition:
        f(w) = int f(t) e^Iwt dt
        In discrete form, it's
        f(j) = dt sum_{i=0}^{N-1} f(i) e^(I 2pi i j/N).
        N is length of the data. 
        domega/2/pi = 1/N/dt
        This corresponds to the inverse FFT in numpy:
            numpy.fft.ifft(a)*2*pi/domega
        """
        if(len(a) != self.N):
            print("MyFFT.Fourier1D: array length error!")
            sys.exit(0)
        else:
            nor = 2.*np.pi/self.dw
            b = np.fft.ifft(a)
            return nor*b

    def iFourier1D(self, a):
        """ 
        1D Fourier transform from w to t using our definition:
        f(t) = int f(w) e^-Iwt dw/2/pi
        In discrete form, it's
        f(i) = dw/2/pi sum_{j=0}^{N-1} f(j) e^(-I 2pi i j/N).
        N is length of the data. 
        domega/2/pi = 1/N/dt
        This corresponds to the FFT in numpy:
            numpy.fft.fft(a)*domega/2/pi
        """
        if(len(a) != self.N):
            print("MyFFT.iFourier1D: array length error!")
            sys.exit(0)
        else:
            nor = self.dw/2/np.pi
            b = np.fft.fft(a)
            return nor*b


np.seterr(over="ignore")


def coth(x):
    """
    hyperbolic cotangent function
    """
    if x == 0.0:
        print("coth:coth(0) is infinity")
        sys.exit(0)
    else:
        return np.cosh(x)/np.sinh(x)


def xcoth(x):
    """
    x*coth(x)
    """
    if x == 0.0:
        return 1.0
    else:
        return x*np.cosh(x)/np.sinh(x)


def bose(w, T):
    """
    bose distribution
    """
    #small = 10e-20
    if T == 0.0:
        if w == 0.0:
            return 1/(np.exp(1.0/U.kb)-1)
        elif w < 0.0:
            return -1.0
        else:
            return 0.0
    else:
        if w == 0.0:
            # return 1/small
            # have problems for finite temperature for bias calculation
            # return 0 seems solves it
            return 0.0
        else:
            return 1.0/(np.exp(w/U.kb/T)-1.0)


def fermi(ep, mu, T):
    """
    fermi distribution
    """
    if T == 0.0:
        if(ep < mu):
            return 1.0
        elif(ep > mu):
            return 0.0
        else:
            return 0.5
    else:
        return 1/(np.exp((ep-mu)/U.kb/T)+1)


def flinterp(x, xs, ys):
    """
    do a linear interpolation of (xs,ys), return the interpolated value at x.
    """
    id = nearest(x, xs)

    # boundaries
    if id == len(xs)-1:
        return ys[-1]
    if id == 0:
        return ys[0]

    # linear interpolation
    dd = xs[id]-x
    if dd < 0:
        return ys[id]+dd/(xs[id]-xs[id-1])*(ys[id]-ys[id-1])
    else:
        return ys[id]+dd/(xs[id]-xs[id+1])*(ys[id]-ys[id+1])


def nearest(b, bs):
    """
    return the index of the element in bs wich is the nearest to b.
    """
    bsn = np.array(bs)
    bst = abs(bsn-b)
    return(list(bst).index(min(bst)))


def rpadleft(bs, b):
    if len(bs) > 1:
        return np.concatenate((np.array([b]), np.array(bs)[:-1]), axis=0)
    elif len(bs) == 1:
        return np.array([b])
    else:
        print("len(bs) is less than 1")
        sys.exit()


def mdot(* args):
    return np.linalg.multi_dot([im for im in args])


def chkShape(a):
    """
    check if a is a n by n matrix, if yes return n
    """
    aa = np.array(a)
    ash = np.shape(aa)
    if(ash[0] == ash[1]):
        return ash[0]
    else:
        print("The matrix should be a n by n matrix")
        sys.exit(0)


def symmetrize(a):
    aa = np.array(a)
    return 0.5*(aa+np.transpose(aa))


def antisymmetrize(a):
    aa = np.array(a)
    return 0.5*(aa-np.transpose(aa))


def dagger(a):
    aa = np.array(a)
    ash = np.shape(aa)
    if(ash[0] != ash[1]):
        print("Not sqaure matrix")
        sys.exit(0)
    return np.transpose(np.conjugate(aa))


def hermitianize(a):
    aa = np.array(a)
    return 0.5*(aa+dagger(aa))


def powerspecq(qs, dt, nmd):
    """
    qs      list of trajectories, shape(nmd,nph)
    dt      time step of MD simulation
    nmd     number of MD steps
    """
    qst = np.transpose(np.array(qs))
    nmd2 = qst.shape[1]
    if nmd != nmd2:
        print("power: qs shape error!")
        sys.exit()
    dw = 2.*np.pi/dt/nmd

    qsw = np.array([myfft(dt, nmd).Fourier1D(a) for a in qst])
    qsw = np.real(np.transpose(qsw*np.conjugate(qsw)))
    return np.array([[i*dw, (dw*i)**2*np.sum(qsw[i])/dt/nmd] for i in range(nmd)])


def powerspecp(ps, dt, nmd):
    """
    ps      list of trajectories, shape(nmd,nph)
    dt      time step of MD simulation
    nmd     number of MD steps
    """
    pst = np.transpose(np.array(ps))
    nmd2 = pst.shape[1]
    if nmd != nmd2:
        print("power: ps shape error!")
        sys.exit()
    dw = 2.*np.pi/dt/nmd

    psw = np.array([myfft(dt, nmd).Fourier1D(a) for a in pst])
    psw = np.real(np.transpose(psw*np.conjugate(psw)))
    return np.array([[i*dw, np.sum(psw[i])/dt/nmd] for i in range(nmd)])



#!/usr/bin/env python
# -*- coding: utf-8 -*

import numpy as np


class sig:
    # This class is used to create a selfenergy object.
    def __init__(self, infile, maxomega, atomgroup0, atomgroup1, dofatomfixed=[[], []], dynmatfile=None, num=1000, eta=0.164e-3):
        # Initializes the selfenergy object.
        # reduced Planck constant unit in: eV*ps
        self.rpc = 6.582119569e-4
        # Boltzmann constant unit in: eV/K
        # self.bc = 8.617333262e-5
        #self.infile = infile
        #self.damp = damp
        self.maxomega = maxomega/self.rpc
        self.intnum = num
        self.eta = eta/self.rpc
        self.dofatomK00 = atomgroup0
        self.dofatomK11 = atomgroup1
        self.dofatomfixed = dofatomfixed
        self.dynmatfile = dynmatfile
        self.ep = np.linspace(0, self.maxomega, self.intnum+1)
        self.getdynmat(infile)
        self.getdk()

    def getdynmat(self, infile):
        from lammps import lammps
        lmp = lammps()
        #lmp = lammps(cmdargs=['-screen', 'none', '-log', 'none'])
        lmp.commands_list(infile)
        self.natoms = lmp.get_natoms()
        box = lmp.extract_box()
        self.boxlo = np.array(box[0])
        self.boxhi = np.array(box[1])
        systype = np.array(lmp.gather_atoms("type", 0, 1))
        mass = lmp.extract_atom("mass", 2)
        self.els = []
        for type in systype:
            self.els.append([mass[type]]*3)
        self.els = np.array(self.els).flatten()
        self.xyz = lmp.gather_atoms("x", 1, 3)
        self.els = np.delete(self.els, self.dofatomfixed[0])
        self.els = np.delete(self.els, [
            dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]])
        self.xyz = np.delete(self.xyz, self.dofatomfixed[0])
        self.xyz = np.delete(self.xyz, [
            dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]])
        if self.dynmatfile is None:
            print('Calculate dynamical matrix')
            lmp.command('dynamical_matrix all eskm 0.000001 file dynmat.dat')
            dynmatdat = np.loadtxt('dynmat.dat')
        else:
            print('Load dynamical matrix from '+str(self.dynmatfile))
            dynmatdat = np.loadtxt(self.dynmatfile)
        lmp.close()
        self.dynmat = []
        self.omegas = []
        self.doffreeatom = 0
        dynlen = int(3*np.sqrt(len(dynmatdat)/3))
        if dynlen != self.natoms*3:
            raise ValueError(
                'System DOF test failed after load dynmat, check again')
        self.dynmat = dynmatdat.reshape((dynlen, dynlen))
        dynmatexcpfixed = np.delete(self.dynmat, self.dofatomfixed[0], axis=0)
        dynmatexcpfixed = np.delete(
            dynmatexcpfixed, self.dofatomfixed[0], axis=1)
        dynmatexcpfixed = np.delete(dynmatexcpfixed, [
            dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]], axis=0)
        dynmatexcpfixed = np.delete(dynmatexcpfixed, [
            dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]], axis=1)
        if len(self.xyz) != len(dynmatexcpfixed):
            raise ValueError(
                'System DOF test failed after atoms reduced, check again')
        print('Calculate angular frequency')
        eigvals, eigvecs = np.linalg.eigh(dynmatexcpfixed)
        ffi = []
        for i, val in enumerate(eigvals):
            if val > 0:
                self.omegas.append(np.sqrt(val)*self.rpc)
            else:
                ffi.append(i)
                # print('False frequency exists in system DOF %i ' %
                #      (i+len(self.dofatomfixed[0])))
                self.omegas.append(-np.sqrt(-val)*self.rpc)
        print('%i false frequencies exist in %i frequencies' %
              (len(ffi), len(self.omegas)))
        np.savetxt('falsefrequencies.dat', ffi, fmt='%d')
        np.savetxt('omegas.dat', self.omegas)
        np.savetxt('eigvecs.dat', eigvecs)

    def getdk(self):
        # Get dynamic matrix of given atoms
        self.K00 = self.dynmat[self.dofatomK00, :][:, self.dofatomK00]
        self.K11 = self.dynmat[self.dofatomK11, :][:, self.dofatomK11]
        self.K01 = self.dynmat[self.dofatomK00, :][:, self.dofatomK11]
        self.K10 = self.dynmat[self.dofatomK11, :][:, self.dofatomK00]
        if np.amax(abs(self.K01 - np.transpose(self.K10)))/np.amax(abs(self.K01)) > 1e-8:
            raise ValueError('Error: K01 and K10 are not symmetric',np.amax(abs(self.K01 - np.transpose(self.K10)))/np.amax(abs(self.K01)))
        else:
            self.K01 = (self.K01+np.transpose(self.K10))/2
            self.K10 = np.transpose(self.K01)

    def sgf(self, omega, direction):
        # Algorithm for surface Green’s function
        if direction == 'R':
            s = self.K00.astype(complex)
            e = self.K11.astype(complex)
            alpha = self.K01.astype(complex)
        elif direction == 'L':
            s = self.K11.astype(complex)
            e = self.K00.astype(complex)
            alpha = self.K10.astype(complex)
        else:
            raise ValueError('Wrong direction, should only be R or L')
        iter = 0
        while np.linalg.norm(alpha) > 1e-8:
            g = np.linalg.inv((omega+self.eta*1j)**2*np.identity(len(e))-e)
            beta = np.transpose(alpha)
            s += np.dot(np.dot(alpha, g), beta)
            e += np.dot(np.dot(alpha, g), beta) + \
                np.dot(np.dot(beta, g), alpha)
            alpha = np.dot(np.dot(alpha, g), alpha)
            iter += 1
            #print('Iteration %i' % iter, 'det(alpha)', np.linalg.norm(alpha))
            if iter >= 100:
                #print('Iteration number for surface Green’s function: %i' % iter)
                raise ValueError(
                    'Iteration number exceeded 100, please increase eta')
        return np.linalg.inv((omega+self.eta*1j)**2*np.identity(len(s))-s)

    def selfenergy(self, omega, direction):
        # self energy
        if direction == 'R':
            return np.dot(np.dot(self.K01, self.sgf(omega, direction)), self.K10)
        elif direction == 'L':
            return np.dot(np.dot(self.K10, self.sgf(omega, direction)), self.K01)
        else:
            raise ValueError('Wrong direction, should only be R or L')

    def gamma(self, Pi):
        return -1j*(Pi-Pi.conjugate().transpose())

    def retargf(self, omega):
        # retarded Green function
        return np.linalg.inv((omega+1e-8*1j)**2*np.identity(len(self.K00))-self.K00-self.selfenergy(omega, 'L')-self.selfenergy(omega, 'R'))

    def tm(self, omega):
        # Transmission
        return np.real(np.trace(np.dot(np.dot(np.dot(self.retargf(omega), self.gamma(self.selfenergy(omega, 'L'))), self.retargf(omega).conjugate().transpose()), self.gamma(self.selfenergy(omega, 'R')))))

    def getse(self, direction):
        # get a set of selfenergy of given omega
        from tqdm import tqdm
        dosx = []
        se = []
        print("Calculate selfenergy of "+str(direction)+" lead")
        for var in tqdm(self.ep, unit="steps", mininterval=1):
            selfenergysplit = self.selfenergy(var, direction)
            se.append(selfenergysplit)
            dosx.append(-np.trace(np.imag(selfenergysplit))*var/np.pi)
        self.dos = np.array(np.column_stack((self.ep, np.array(dosx))))
        np.savetxt('densityofstates_'+str(direction)+'.dat', np.column_stack(
            (self.dos[:, 0]*self.rpc, self.dos[:, 1])))
        return np.array(se)

    def gettm(self):
        # get a set of transmission of given omega
        from tqdm import tqdm
        tm = []
        print("Calculate transmission")
        for var in tqdm(self.ep, unit="steps", mininterval=1):
            tm.append(self.tm(var))
        self.tmnumber = np.array(np.column_stack((self.ep, np.array(tm))))
        np.savetxt('transmission.dat', np.column_stack(
            (self.tmnumber[:, 0]*self.rpc, self.tmnumber[:, 1])))
        # return np.array(se)

    def plotresult(self, lines=180):
        from matplotlib import pyplot as plt
        plt.figure(0)
        plt.hist(self.omegas, bins=lines)
        plt.xlabel('Frequence(eV)')
        plt.ylabel('Number')
        #plt.xlim(0, self.maxomega*self.rpc)
        plt.savefig('omegas.png')
        plt.figure(1)
        plt.plot(self.dos[:, 0]*self.rpc, self.dos[:, 1])
        plt.xlabel('Frequence(eV)')
        plt.ylabel('DOS')
        plt.gca().ticklabel_format(style='sci', scilimits=(-1, 2), axis='y')
        plt.savefig('densityofstates.png')
        plt.figure(2)
        plt.plot(self.tmnumber[:, 0]*self.rpc, self.tmnumber[:, 1])
        plt.xlabel('Frequence(eV)')
        plt.ylabel('Transmission')
        plt.savefig('transmission.png')


if __name__ == '__main__':
    import time

    #from sclmd.selfenergy import sig
    infile = [
        'atom_style full',
        'units  metal',
        #'boundary p p p',
        'read_data sig.data',
        'pair_style rebo',
        'pair_coeff *   *   CH.rebo C',
        #'min_style  cg',
        'minimize   1e-25   1e-25   5000    10000',
        'dump   1   all xyz 1   dump.xyz',
        'run    0',
    ]
    time_start = time.time()
    atomgroup0 = range(24*3, 32*3)
    atomgroup1 = range(32*3, 40*3)
    mode = sig(infile, 0.12, atomgroup0, atomgroup1,
               dofatomfixed=[[], []], dynmatfile=None, num=2000, eta=0.164e-3)
    mode.getse('L')
    mode.getse('R')
    mode.gettm()
    mode.plotresult()
    print('time cost', time.time()-time_start, 's')



#!/usr/bin/env python
# -*- coding: utf-8 -*

import os
import sys
import time

import numpy as np
from netCDF4 import Dataset
from numpy import linalg as LA
from tqdm import tqdm

import sclmd.units as U
from sclmd.functions import bose, chkShape, mdot, powerspecp, rpadleft, symmetrize


class md:
    """
    do langevin molecular dynamics using a modified velocity verlet method.
    The friction from both electrons and phonons are time-local now.

    The following variables are initialied when called:
        xyz         position array of atoms [x1,y1,z1,x2,y2,z2,...]
        els         list of elements ["Au","Au",...]
        nta         total number of atoms, including system and buffering atoms
        syslist     list made from indices of system atoms (python indices,
                    starting from 0)
        dt          time step 
        nmd         len of md simulation
        na          number of system atoms
        nph         number of system degrees of freedom
        dyn         dynamical matrix of the system unit in eV^2
        harmonic    how to calculate the potential force: harmonic approximation
                    or siesta
        T           Average temperature unit in K
        saveq       whether to save md trajectories
        savep       whether to save md trajectories
        ml          length of memory kernel (max(all baths))

        hw          eigen frequencies
        U           eigen vectors, each column corresponds to one eigen vector,
                    for example, hw, U = np.linalg.eigh(DynMat)

        t,p,q       md time step, velocity and position vector at t

        q0,f0       q and f of previous potforce run

        baths       List of baths connecting to the system
        fbaths      Force from all the baths at time t
        etot        total energy at each time step
        nstep       Output atomic position after nstep MD 
        saveall     Whether to save all information to .nc file
        rmnc        Remove NC files after calculation
    """

    def __init__(self, dt, nmd, T, syslist=None, axyz=None, dyn=None, nstart=0, nstop=1, npie=1, md2ang=0.06466):
        self.nstart = nstart
        self.nstop = nstop
        self.dt, self.nmd = dt, nmd
        # self.harmonic = harmonic
        self.T = T
        self.npie = npie
        self.saveall = False
        self.savep = False
        self.saveq = False
        self.rmnc = False
        #self.mixf = False
        self.nstep = None
        self.pforce = None
        #self.pforce2 = None
        self.constraint = None
        self.atomlist = None
        # var: xyz,nta,els
        self.SetXyz(axyz)
        # var: syslist,na,nph
        if syslist is not None:
            if(len(syslist) > self.nta or min(syslist) < 0 or
               max(syslist) > self.nta-1):
                print("syslist out of range")
                sys.exit(0)
            else:
                self.syslist = np.array(syslist, dtype='int')
            # number of system atoms
            self.na = len(syslist)
            # number of system degrees of freedom
            self.nph = 3*len(syslist)
        elif axyz is not None:
            # set using axyz
            # all are system atoms
            self.syslist = np.array(range(len(axyz)), dtype='int')
            self.na = len(self.syslist)
            self.nph = 3*len(self.syslist)
        else:
            self.syslist = None
            self.na = None
            self.nph = None

        self.ml = 1
        self.cf = 0
        self.t = 0
        self.p = []
        self.q = []
        self.pinit = []
        self.qinit = []
        self.q0 = []
        self.f0 = []

        #qhis and phis

        # list of baths
        self.baths = []
        self.fhis = []
        self.fbaths = []

        self.etot = np.zeros(nmd)

        # vars: dyn,hw,U,nph
        self.initranvel = True
        self.setDyn(dyn)

        # var: ps,qs,power
        # self.ResetSavepq()
        self.md2ang = md2ang
        self.mass = []
        self.get_atommass()
        if len(self.mass) != len(self.els):
            print("Wrong setting in els or mass")
            sys.exit(0)
        self.conv = self.md2ang*np.array([3*[1.0/np.sqrt(mass)]
                                          for mass in self.mass]).flatten()

    def get_atommass(self):
        for atomsname in self.els:
            for key, value in list(U.AtomicMassTable.items()):
                if atomsname == key:
                    self.mass.append(value)

    def info(self):
        print("--------------------------------------------")
        print("Basis information of the MD simulation:")
        # print("Harmonic force: "+str(self.harmonic))
        print("System atom number:"+str(self.na))
        print("MD time step:"+str(self.dt))
        print("MD number of steps:"+str(self.nmd))
        print("MD memory kernel length:"+str(self.ml))
        print("Number of baths attached:"+str(len(self.baths)))
        print("md.save all momentum: %r" % self.savep)
        print("md.save all postions: %r" % self.saveq)
        # if self.dyn is None:
        #    print("md.info: No dynamical matrix input")
        # sys.exit()

    def ResetSavepq(self):
        if self.savep and self.nmd is not None and self.nph is not None:
            self.ps = np.zeros((self.nmd, self.nph))
        if self.saveq and self.nmd is not None and self.nph is not None:
            self.qs = np.zeros((self.nmd, self.nph))

    # def energy(self):
    #    return 0.5*mdot(self.p,self.p)+0.5*mdot(self.q,self.dyn,self.q)
    def energy(self):
        """
        kinetic energy
        """
        return 0.5*mdot(self.p, self.p)

    def AddBath(self, bath):
        """
        Adding a bath
        """
        if self.dt != bath.dt:
            print("md.AddBath: md time step dt not consistent")
            sys.exit()
        if self.nmd != bath.nmd:
            print("md.AddBath: number of md steps nmd not consistent")
            sys.exit()
        self.baths.append(bath)
        # make sure we save enought memory for all the baths
        if(bath.ml > self.ml):
            self.ml = bath.ml
        self.fbaths.append(np.zeros(self.nph))
        # force history
        self.fhis.append(np.zeros((self.nmd, self.nph)))

    def AddPowerSection(self, atomlist):
        self.atomlist = atomlist
        self.poweratomlist = np.empty((len(self.atomlist), self.nmd, 2))

    def AddConstr(self, constr):
        self.constraint = constr

    def CalPowerSpec(self, cal=True):
        self.savep = cal
        self.power = np.empty((self.nmd, 2))

    def CalAveStruct(self, cal=True):
        self.saveq = cal

    def SaveAll(self, save=True):
        self.saveall = save

    def SaveTraj(self, nstep=100):
        self.nstep = nstep

    def RemoveNC(self, rmnc=True):
        self.rmnc = rmnc

    def SetT(self, T):
        self.T = T

    def SetMD(self, dt, nmd):
        self.dt, self.nmd = dt, nmd
        self.etot = np.zeros(nmd)

    # def SetHarm(self, harmonic):
    #     self.harmonic = harmonic

    def noranvel(self, rf=False):
        self.initranvel = rf

    def SetXyz(self, axyz):
        if axyz is not None:
            print("md.SetXyz:Seting xyz and nta")
            self.xyz = np.array([a[1:] for a in axyz], dtype='d').flatten()
            self.els = [a[0] for a in axyz]
            self.nta = len(axyz)
        else:
            self.xyz = None
            self.els = None
            self.nta = None

    def SetSyslist(self, syslist):
        print("md.SetXyz:Seting syslist")
        self.syslist = np.array(syslist)
        # number of system atoms
        self.na = len(syslist)
        # number of system degrees of freedom
        self.nph = 3*len(syslist)
        if self.xyz is not None:
            if len(self.syslist) > self.nta:
                print("md.SetSyslist:system atom number larger than total atom number")
                sys.exit()

    def setDyn(self, dyn=None):
        """
        set up the dynamical matrix of the system
        """
        if dyn is not None:
            print("md.setDyn: setting dynamical matrix")
            ndyn = np.array(dyn)
            #print("md.setDyn: checking dynamical matrix")
            n = chkShape(ndyn)
            if self.nph is not None and self.nph != n:
                print("md.setDyn: the dimension of dynamical matrix is wrong")
                sys.exit(0)
            self.nph = n
            #print("md.setDyn: symmetrizing dynamical matrix")
            self.dyn = symmetrize(ndyn)

            av, au = LA.eigh(self.dyn)
            if min(av) < 0:
                avn = 0.*av
                conut = 0
                for i in range(len(av)):
                    if av[i] < 0:
                        avn[i] = 0.
                        conut += 1
                    else:
                        avn[i] = av[i]
                av = avn
                print("md.setDyn: "+str(conut)+" negative frequencies removed")
            self.hw = np.array(list(map(np.real, list(map(np.sqrt, av)))))
            self.U = np.array(au)
            self.dyn = mdot(self.U, np.diag(
                np.array(av)), np.transpose(self.U))
            # if min(av)>=0:
            #    print "the dynmat should not change much"
            #    print "max diff. of dynmatrix:", abs(self.dyn-ndyn).sum()
            print("md.setDyn: Done")
        else:
            self.dyn = None
            self.hw = [1.0]
            self.U = None
            print("md.setDyn: no dynamical matrix provided")
            print("Set max eigen frequencies 1.0eV")
            # sys.exit()

    def initialise(self):
        """
        initial displacement and velocity from the dynamical matrix
        """
        self.t = 0
        if self.dyn is None:
            print("md.initial: no dynamical matrix")
            # sys.exit()
            print("p,q set to 0")
            self.p = np.zeros(self.nph)
            self.q = np.zeros(self.nph)
            self.pinit = np.zeros(self.nph)
            self.qinit = np.zeros(self.nph)
        else:
            av = self.hw
            au = self.U

            dis = np.zeros(len(av))
            vel = np.zeros(len(av))
            for i in range(len(av)):
                # cutoff energy 0.005 eV
                # do not initialise motion due to slow modes
                # because it may gives large displacement
                if av[i] < 0.01:
                    am = 0.0
                else:
                    am = ((bose(av[i], self.T)+0.5)*2.0/av[i])**0.5
                r = np.random.rand()
                dis = dis + au[:, i]*am*np.cos(2.*np.pi*r)
                vel = vel - av[i]*au[:, i]*am*np.sin(2.*np.pi*r)

                dis = ApplyConstraint(dis, self.constraint)
                vel = ApplyConstraint(vel, self.constraint)

            if not self.initranvel:
                print("Not set init displacement & velocity")
                self.p = np.zeros(self.nph)
                self.pinit = np.zeros(self.nph)
                self.q = np.zeros(self.nph)
                self.qinit = np.zeros(self.nph)
            else:
                self.p = vel
                self.pinit = vel
                self.q = dis
                self.qinit = dis

    def ResetHis(self):
        """
        set history list of the friction kernel to zeros
        """
        if self.nph is not None and self.ml is not None:
            self.qhis = np.zeros((self.ml, self.nph))
            self.phis = np.zeros((self.ml, self.nph))
        else:
            print("self.nph and self.ml are not set")
            sys.exit()

    def GetPower(self):
        """
        calculate the power spectrum from the MD trajectories.
        """
        print("md.GetPower: generate power spectrum from trajectories.")
        self.power = powerspecp(self.ps, self.dt, self.nmd)
        if self.atomlist is not None:
            for layers in range(len(self.atomlist)):
                self.poweratomlist[layers] = powerspecp(
                    self.ps[:, self.atomlist[layers]], self.dt, self.nmd)

    def CompareForce(self,dynmat):
        self.cf = 1
        self.cfdynmat = dynmat
        self.cflist = []

    def vv(self, id):
        """
        velocity-verlet method integrator
        """
        # print "velocity-verlet integrator"
        t, p, q = self.t, self.p, self.q
        t = int(t)
        if self.savep:
            self.ps[t % self.nmd] = p
        if self.saveq:
            self.qs[t % self.nmd] = q
        if self.cf:
            self.cflist.append(self.potforce(q) -(-1*mdot(self.cfdynmat, q)))

        # total energy
        #self.etot = np.append(self.etot,self.energy())
        self.etot[t % self.nmd] = self.energy()

        # update history here
        self.qhis = rpadleft(self.qhis, q)
        self.phis = rpadleft(self.phis, p)

        # calculate displacement at next time
        f = self.force(t, p, q, 0)
        pthalf = p + f*self.dt/2.0
        qtt = q + p*self.dt + f*self.dt**2/2.0

        # evaluate current
        for i in range(len(self.baths)):
            # self.baths[i].cur=np.append(self.baths[i].cur,mdot(self.fbaths[i],p))
            self.baths[i].cur[t % self.nmd] = mdot(self.fbaths[i], p)
            self.fhis[i][t % self.nmd] = self.fbaths[i]

        # calculate velocity at next time
        f = self.force(t, pthalf, qtt, 1)
        ptt1 = pthalf+self.dt*f/2.0
        f = self.force(t, ptt1, qtt, 1)
        ptt2 = pthalf+self.dt*f/2.0

        # constraint
        ptt2 = ApplyConstraint(ptt2, self.constraint)
        qtt = ApplyConstraint(qtt, self.constraint)

        t = t+1
        self.t, self.p, self.q, self.f = t, ptt2, qtt, f

    def force(self, t, p, q, id=0):
        """
        force due to the dynamical matrix
        """
        it = t+id

        # print "potential force"
        pf = self.potforce(q)

        # apply constraint
        # pf=ApplyConstraint(pf,self.constraint)

        # print "friction and random force "
        if id == 0:
            tphis = self.phis
            tqhis = self.qhis
        else:
            tphis = rpadleft(self.phis, p)
            tqhis = rpadleft(self.qhis, q)
        for i in range(len(self.baths)):
            self.fbaths[i] = self.baths[i].bforce(it, tphis, tqhis)
            pf = pf + self.fbaths[i]
        return pf

    def potforce(self, q):
        """
        Potential force from drivers
        Now we have the following 4 drivers:
            Siesta, Brenner, Lammps, harmonic

        q is an array of displacements of the atoms in self.syslist
        """
        # TODO Simplified code
        # self.q0 and f0 are the displacment and
        # force of last call. If the displacement
        # did not change, use the old force.
        if sameq(q, self.q0):
            return self.f0

        # if len(q)/3 != len(self.syslist):
        #     print("md:potforce: length error")
        #     sys.exit()

        # search for possible drivers
        # if self.mixf and self.pforce is not None and self.dyn is not None:
        #    fdyn = -1*mdot(self.dyn, q)
        #    fpot2 = self.pforce2.force(q[self.atomtomix[1]])
        #    fpot = -1*mdot(self.dyn[self.atomtomix[1], :][:, self.atomtomix[0]], q[self.atomtomix[0]])+fpot2-1*mdot(self.dyn[self.atomtomix[1], :][:, self.atomtomix[2]], q[self.atomtomix[2]])
        #    f = np.append(np.append(fdyn[self.atomtomix[0]], fpot), fdyn[self.atomtomix[2]])
        # elif self.pforce is not None:
        if self.pforce is not None:
            f = self.pforce.force(q)
        # use dynamical matrix
        elif self.dyn is not None:
            f = -1*mdot(self.dyn, q)
        else:
            print("no driver, no md")
            sys.exit()
        # save
        self.q0 = q
        self.f0 = f
        return f

    # def mixforce(self, atomtomix=None):
    #    print("force mixed")
    #    self.mixf = True
    #    self.atomtomix = atomtomix

    def AddPotential(self, pint):
        """
        add siesta, lammps or Brenner instance
        """
        self.pforce = pint

    # def AddAnotherPotential(self, ppint):
    #    """
    #    add siesta, lammps or Brenner instance
    #    """
    #    self.pforce2 = ppint

    def Run(self):
        """
        define nrep,npie
        """
        # initialise t,p,q
        # if find old unfinished data,
        # they are over-written
        self.initialise()
        # reset qhis,phis
        self.ResetHis()
        self.info()

        # loop over independent md runs
        for j in range(self.nstart, self.nstop):
            print("\n"+"MD run: "+str(j))
            fn = "MD"+str(j)+".nc"
            fnm = "MD"+str(j-1)+".nc"

            if os.path.isfile(fn):
                print("find file: "+fn)
                ipie = int(ReadNetCDFVar(fn, 'ipie')[0])
                if(ipie+1 < self.npie):
                    print("unfinished run")
                    print("reading resume information")
                    self.p = ReadNetCDFVar(fn, 'p')
                    self.q = ReadNetCDFVar(fn, 'q')
                    self.t = ReadNetCDFVar(fn, 't')[0]
                    self.qhis = ReadNetCDFVar(fn, 'qhis')
                    self.phis = ReadNetCDFVar(fn, 'phis')
                    if self.savep:
                        self.power = ReadNetCDFVar(fn, 'power')
                        if self.atomlist is not None:
                            self.poweratomlist = ReadNetCDFVar(
                                fn, 'poweratomlist')
                    if self.saveall and self.saveq and self.savep:
                        self.qs = ReadNetCDFVar(fn, 'qs')
                        self.ps = ReadNetCDFVar(fn, 'ps')
                    else:
                        print("saveall savep & saveq need to be set true to continue")
                        sys.exit(0)
                    for i in range(len(self.baths)):
                        self.baths[i].noise = ReadNetCDFVar(fn, 'noise'+str(i))

                elif(ipie+1 == self.npie):
                    print("finished run")
                    if self.savep:
                        self.power = ReadNetCDFVar(fn, 'power')
                        if self.atomlist is not None:
                            self.poweratomlist = ReadNetCDFVar(
                                fn, 'poweratomlist')
                    self.t = ReadNetCDFVar(fn, 't')[0]
                    continue
                else:
                    print("ipie error")
                    print("ipie=", ipie)
                    sys.exit()
            else:
                print("new run")
                ipie = -1  # yes,-1
                if os.path.isfile(fnm):
                    print("reading history from previous run")
                    self.p = ReadNetCDFVar(fnm, 'p')
                    self.q = ReadNetCDFVar(fnm, 'q')
                    qhis0 = ReadNetCDFVar(fnm, 'qhis')
                    phis0 = ReadNetCDFVar(fnm, 'phis')
                    self.t = ReadNetCDFVar(fnm, 't')[0]
                    if qhis0.shape == self.qhis.shape and\
                            phis0.shape == self.phis.shape:
                        self.qhis = qhis0
                        self.phis = phis0
                elif j == 0:
                    print("initialize a new simulation")
                else:
                    print("no previous nc file exists")
                    sys.exit()
                # noise generation
                for i in range(len(self.baths)):  # loop over baths
                    self.baths[i].gnoi()
                    # print np.shape(self.baths[i].noise)
                    # stppp

                # reset qs and ps to zero
                self.ResetSavepq()

            # loop over md steps
            ipie1 = ipie+1
            iss = ipie1+np.array(range(self.npie-ipie1))
            trajfile = open('trajectories'+"."+str(self.T) +
                            "."+"run"+str(j)+'.ani', 'w')
            for i in iss:
                print("Progress of MD")
                for _ in tqdm(range(int(self.nmd/self.npie)), unit="steps", mininterval=1):
                    self.vv(j)
                    if self.nstep is not None and ((self.t-1) == 0 or (self.t-1) % self.nstep == 0):
                        #head = str(len(self.els))+'\n'+str(self.t-1)+'\n'
                        # np.savetxt(trajfile, np.column_stack((
                        #    self.els, np.transpose(np.reshape(self.xyz+self.conv*self.q,(3,len(self.els))))[:], np.transpose(np.reshape(self.f,(3,len(self.els))))[:])), header=head)
                        trajfile.write(str(len(
                            self.els))+'\n'+str(self.t-1)+'\n')
                        structure = self.xyz+self.conv*self.q
                        for ip in range(len(self.els)):
                            trajfile.write(str(self.els[ip])+'    '+str(structure[ip*3])+'   '+str(structure[ip*3+1])+'   '+str(
                                structure[ip*3+2])+'   '+str(self.f[ip*3])+'   '+str(self.f[ip*3+1])+'   '+str(self.f[ip*3+2])+'\n')
                self.dump(i, j)
            trajfile.close()

            if self.cf:
                np.save("deltaforce"+".run"+str(j), self.cflist)
                self.cflist = []
                
            if self.savep:
                # power spectrum
                power = np.copy(self.power)
                if self.atomlist is not None:
                    poweratomlist = [None]*len(self.atomlist)
                    for layers in range(len(self.atomlist)):
                        poweratomlist[layers] = np.copy(
                            self.poweratomlist[layers])
                self.GetPower()
                self.power = (power*(j-self.nstart)+self.power) / \
                    float(j-self.nstart+1)
                if self.atomlist is not None:
                    for layers in range(len(self.atomlist)):
                        self.poweratomlist[layers] = (
                            poweratomlist[layers]*(j-self.nstart)+self.poweratomlist[layers])/float(j-self.nstart+1)
                # power spectrum
                f = open("power."+str(self.T)+"."+"run"+str(j)+".dat", "w")
                # f.write("#k-point averaged transmission and DoS from MAMA.py\n")
                # f.write("#energy    transmission    DoSL    DoSR\n")
                for ni in range(len(self.power)):
                    # only write out power spectrum upto 1.5max(hw)
                    if self.hw is not None:
                        if(self.power[ni, 0] < 1.5*max(self.hw)):
                            f.write("%f     %f \n" %
                                    (self.power[ni, 0], self.power[ni, 1]))
                        else:
                            break
                    else:
                        f.write("%f     %f \n" %
                                (self.power[ni, 0], self.power[ni, 1]))
                f.close()
                if self.atomlist is not None:
                    for layers in range(len(self.atomlist)):
                        # power spectrum atomlist
                        f = open("poweratomlist."+str(layers)+"." +
                                 str(self.T)+"."+"run"+str(j)+".dat", "w")
                        # f.write("#k-point averaged transmission and DoS from MAMA.py\n")
                        # f.write("#energy    transmission    DoSL    DoSR\n")
                        for ni in range(len(self.poweratomlist[layers])):
                            # only write out power spectrum upto 1.5max(hw)
                            if self.hw is not None:
                                if(self.poweratomlist[layers][ni, 0] < 1.5*max(self.hw)):
                                    f.write("%f     %f \n" % (
                                        self.poweratomlist[layers][ni, 0], self.poweratomlist[layers][ni, 1]))
                                else:
                                    break
                            else:
                                f.write("%f     %f \n" % (
                                    self.poweratomlist[layers][ni, 0], self.poweratomlist[layers][ni, 1]))
                        f.close()
            # dump again, to make sure power is all right
            self.dump(i, j)

            # heat current
            for ii in range(len(self.baths)):
                fk = open("kappa."+str(self.T)+"."+"bath" +
                          str(ii)+".run"+str(j)+".dat", "w")
                # write average current
                fk.write("%i %f    %f \n" %
                         (j, self.T, np.mean(self.baths[ii].cur)*U.curcof))
                fk.close()
            if self.saveq:
                # save average structure
                f = open("avestructure."+str(self.T) +
                         "."+"run"+str(j)+".dat", "w")
                avestructure = self.conv*(self.qs.mean(axis=0)) + self.xyz
                f.write(str(len(
                    self.els))+'\n'+"average structure"+'\n')
                for ip in range(len(self.els)):
                    f.write(str(self.els[ip])+'    '+str(avestructure[ip*3])+'   '+str(
                        avestructure[ip*3+1])+'   '+str(avestructure[ip*3+2])+'\n')
                f.close()

            if self.rmnc:
                if os.path.exists("MD"+str(j-1)+".nc"):
                    print("Remove MD"+str(j-1)+".nc")
                    os.remove("MD"+str(j-1)+".nc")
                # else:
                #    print("No NC file exists.")

    def dump(self, ipie, id):
        """
        dump md information
        """
        outfile = "MD"+str(id)+".nc"
        NCfile = Dataset(outfile, 'w', 'Created '+time.ctime(time.time()))
        NCfile.title = 'Output from md.py'
        NCfile.createDimension('nph', self.nph)
        # NCfile.createDimension('na',self.na)
        NCfile.createDimension('one', 1)
        NCfile.createDimension('two', 2)
        NCfile.createDimension('mem', self.ml)
        NCfile.createDimension('nmd', self.nmd)
        NCfile.createDimension('nnmd', None)
        if self.atomlist is not None:
            NCfile.createDimension('atomlist', len(self.atomlist))

        for i in range(len(self.baths)):
            NCfile.createDimension('n'+str(i), self.baths[i].nc)

        # els
        # Write2NetCDFFile(NCfile,self.els,'elements',('na',),units='')

        if self.saveall:
            # noise series
            for i in range(len(self.baths)):
                # NCfile.createDimension('n'+str(i),self.baths[i].nc)
                Write2NetCDFFile(NCfile, self.baths[i].noise, 'noise'+str(i),
                                 ('nnmd', 'n'+str(i),), units='')
                Write2NetCDFFile(
                    NCfile, self.fhis[i], 'fhis'+str(i), ('nnmd', 'nph',), units='')
            # save all the histories of p,q or not
            if self.savep:
                Write2NetCDFFile(NCfile, self.ps, 'ps',
                                 ('nnmd', 'nph',), units='')
            if self.saveq:
                Write2NetCDFFile(NCfile, self.qs, 'qs',
                                 ('nnmd', 'nph',), units='')
        if self.savep:
            # power spectrum
            Write2NetCDFFile(NCfile, self.power, 'power',
                             ('nnmd', 'two',), units='')
            if self.atomlist is not None:
                Write2NetCDFFile(NCfile, self.poweratomlist,
                                 'poweratomlist', ('atomlist', 'nnmd', 'two'), units='')

        # energy
        Write2NetCDFFile(NCfile, self.etot, 'energy', ('nnmd',), units='')
        # velocity
        Write2NetCDFFile(NCfile, self.p, 'p', ('nph',), units='')
        # displacement
        Write2NetCDFFile(NCfile, self.q, 'q', ('nph',), units='')
        # current time
        Write2NetCDFFile(NCfile, self.t, 't', ('one',), units='')
        # current segment
        Write2NetCDFFile(NCfile, ipie, 'ipie', ('one',), units='')
        # memory kernel for p
        Write2NetCDFFile(NCfile, self.phis, 'phis', ('mem', 'nph',), units='')
        # memory kernel for q
        Write2NetCDFFile(NCfile, self.qhis, 'qhis', ('mem', 'nph',), units='')

        NCfile.close()


# misc driver routines
def Write2NetCDFFile(file, var, varLabel, dimensions, units=None, description=None):
    # print 'Write2NetCDFFile:', varLabel, dimensions
    tmp = file.createVariable(varLabel, 'd', dimensions, zlib=True)
    tmp[:] = var
    if units:
        tmp.units = units
    if description:
        tmp.description = description


def ReadNetCDFVar(file, var):
    print("ReadNetCDFFile: reading " + var)
    f = Dataset(file, 'r')
    vv = np.array(f.variables[var])
    f.close()
    return vv


def sameq(q1, q2):
    if(len(q1) != len(q2)):
        # print "sameq: qq1 and qq2 not same length"
        return False
    qq1 = np.array(q1)
    qq2 = np.array(q2)
    dif = qq1-qq2
    # if the difference is less than 10e-10
    # use the old force
    if(max(abs(dif)) < 10e-10):
        return True
    else:
        return False


def ApplyConstraint(f, constr=None):
    """
    apply constraint to the force

    constr is an array of vectors
    This subroutine zerofy the force along each vector
    """
    if constr is None:
        return f
    nf = np.array(f)*1.0
    for i in range(len(constr)):
        nf[constr[i]] = 0
    return nf


if __name__ == "__main__":
    import time
    import numpy as np
    from sclmd.baths import ebath
    from sclmd.tools import calHF, calTC
    from sclmd.lammpsdriver import lammpsdriver
    from sclmd.md import md

    lammpsinfile = [
        #"log none",
        "units metal ",
        "dimension 3 ",
        "boundary f p p",
        "atom_style full",
        "read_data structure.data",
        "pair_style rebo ",
        "pair_coeff * * CH.rebo C H",
    ]
    # temperature
    T = 300
    delta = 0.1
    nstart = 0
    nstop = 2
    # time = 0.658fs #time unit
    dt = 0.25/0.658
    # number of md steps
    nmd = 2**10
    # initialise lammps run
    lmp = lammpsdriver(infile=lammpsinfile)
    time_start = time.time()

    print("initialise md")
    fixatoms = [range(0*3, (19+1)*3), range(181*3, (200+1)*3)]

    # print(("constraint:",constraint))
    # Molecular Junction atom indices
    slist = range(70*3, (130+1)*3)
    cutslist = [range(70*3, (89+1)*3),
                range(90*3, (109+1)*3), range(110*3, (130+1)*3)]
    # atom indices that are connecting to debyge bath
    ecatsl = range(20*3, (69+1)*3)
    ecatsr = range(131*3, (180+1)*3)

    # if slist is not given, md will initialize it using xyz
    mdrun = md(dt, nmd, T, axyz=lmp.axyz,
               nstart=nstart, nstop=nstop)
    # attache lammps driver to md
    mdrun.AddPotential(lmp)
    # unit in 0.658211814201041 fs
    damp = 100/0.658211814201041

    etal = (1.0/damp)*np.identity(len(ecatsl), np.float)
    etar = (1.0/damp)*np.identity(len(ecatsr), np.float)
    # atom indices that are connecting to bath
    ebl = ebath(ecatsl, T*(1+delta/2), mdrun.dt, mdrun.nmd,
                wmax=1., nw=500, bias=0.0, efric=etal, classical=False, zpmotion=True)
    mdrun.AddBath(ebl)
    ebr = ebath(ecatsr, T*(1-delta/2), mdrun.dt, mdrun.nmd,
                wmax=1., nw=500, bias=0.0, efric=etar, classical=False, zpmotion=True)
    mdrun.AddBath(ebr)

    mdrun.AddConstr(fixatoms)
    # mdrun.noranvel()
    # mdrun.CalPowerSpec()
    # mdrun.AddPowerSection([ecatsl, slist, ecatsr])
    # mdrun.CalAveStruct()
    mdrun.SaveTraj()
    # mdrun.RemoveNC()

    mdrun.Run()
    lmp.quit()
    calHF()
    calTC(delta=delta, dlist=0)
    time_end = time.time()
    print('time cost', time_end-time_start, 's.')



#!usr/bin/env python
# -*- coding:utf-8 _*-
from setuptools import setup, find_packages

VERSION = '0.3.9'

setup(
    name='sclmd',
    version=VERSION,
    description='Semi-classical Langevin molecular dynamics',
    long_description='Molecular dynamics script using a semi-classical generalized Langevin equation',
    keywords='md molecular dynamics quantum Langevin',
    #author='Jing-Tao Lv',
    #author_email='jtlu@hust.edu.cn',
    #url='https://github.com/phyjtlu/pymd',
    license='GNU General Public License v3.0',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        "numpy>=1.8.2",
        "netCDF4",
        "tqdm",
    ],
    python_requires='>=3.7',
    #entry_points={
    #    'console_scripts': [
    #        'sclmd = sclmd.main:main'
    #    ]
    #}
)


#!/usr/bin/env python
# -*- coding: utf-8 -*

import sys

import numpy as np
from numpy import linalg as LA
from tqdm import tqdm

import sclmd.units as U
from sclmd.functions import bose, chkShape, flinterp, hermitianize, mdot, myfft


# --------------------------------------------------------------------------------------
def mf(f, cats, lens):
    """
    padding f to dimension len
    """
    t = np.zeros(lens)
    for i in range(len(cats)):
        t[cats[i]] = f[i]
    return t
# --------------------------------------------------------------------------------------

# phonon noise in w space


def phnoisew(gamma, wl, T, phcut, classical=False, zpmotion=True):
    """
    gamma       gamma[w]=-Im[Pi^r[w]]/w
    wl          energy points of gamma (only the positive part is needed)
    T           temperature of phonon bath
    phcut       cutoff energy of phonon bath

    The noise spectrum in w space is: 2 w gamma(w) (bose(w,T)+0.5)
    20180816: equ() already gives 2*w*(bose(w,T)+zpmotion)
    """
    n = len(wl)
    gamma = np.array(gamma)
    noiw = 0.0*gamma

    for i in range(n):
        #noiw[i] = equ(wl[i],phcut,T,classical,zpmotion)*wl[i]*gamma[i]
        # 20180816 - equ() already has w included.
        noiw[i] = equ(wl[i], phcut, T, classical, zpmotion)*gamma[i]
    return noiw


# phonon noise generator
def phnoise(gamma, wl, T, phcut, dt, nmd, classical=False, zpmotion=True):
    """
    gamma       gamma[w]=-Im[Pi^r[w]]/w
    wl          energy points of gamma (only the positive part is needed)
    T           temperature of phonon bath
    phcut       cutoff energy of phonon bath
    dt          time step of md 
    nmd         number of md steps


    The noise spectrum in w space is: 2 w gamma(w) (bose(w,T)+0.5)
    """
    print("phnoise: generate phonon noise from the correlation function")

    hlen = int(nmd/2)
    dw = 2.0*np.pi/dt/nmd
    delta = dt*nmd  # Dirac delta delta

    nx = np.shape(np.array(gamma[0]))[0]

    # positive frequency
    phnoi1 = []
    print("Progress of phonon noise generate")
    for i in tqdm(range(hlen+1), unit="steps", mininterval=1):
        w = dw*i
        # flinterp: linear interpolation of gamma
        # equ(w) = 2.0*(bose(w,T)+0.5)
        amat = delta*equ(w, phcut, T, classical, zpmotion) * \
            flinterp(w, wl, gamma)
        amat = hermitianize(amat)
        # eigh only takes half of the matrix,
        # always return real eigenvalues and vectors
        av, au = LA.eigh(amat)
        # generate multivariance gaussian random numbers
        phnoi1.append(vargau(av, au))
    phnoi1 = np.array(phnoi1)

    # negative frequency
    phnoi2 = []
    for i in range(hlen):
        phnoi2.append(np.conjugate(phnoi1[hlen-i, :]))
    phnoi2 = np.array(phnoi2)

    phnoi1 = np.delete(phnoi1, -1, 0)  # delete last row
    phnoi = np.concatenate((phnoi1, phnoi2), axis=0)

    phnoit = []
    fti = myfft(dt, nmd)
    for i in range(nx):
        phnoit.append(fti.iFourier1D(phnoi[:, i]))  # w->t
    return np.transpose(np.array(phnoit))

# electron noise in w space


def enoisew(wl, efric, exim, exip, bias, T, ecut, classical=False, zpmotion=True):
    """
    NO SHAPE CHECKING HERE

    shape of efric,exim,exip (md.nph,md.nph)

    wl          list of frequencies(energies) where the noise is calculated
    efric       electronic friction coefficient
    exim,exip   Im,Re[MA_LMA_R]
    bias        muL-muR
    T           temperature
    ecut        cutoff energy
    """
    nw = len(wl)
    # shape checking
    nc = chkShape(efric)
    nm = chkShape(exim)
    np = chkShape(exip)

    if nc != nm or nc != np:
        print("enoisew: efric shape error!")
        # stoppp

    enoi1 = np.zeros((nw, nc, nc), np.complex)
    for i in range(nw):
        w = wl[i]
        # equilibrium part
        aw = equ(w, ecut, T, classical, zpmotion)
        amate = aw*np.array(efric)
        # nonequilibrium minus part
        awm = equ(U.hbar*w-bias, ecut, T, classical, zpmotion)
        amatm = -0.5*aw*np.array(exip)+0.5*awm * \
            (np.array(exip)+1j*np.array(exim))  # yes,+
        # nonequilibrium plus part
        awp = equ(U.hbar*w+bias, ecut, T, classical, zpmotion)
        amatp = -0.5*aw*np.array(exip)+0.5*awp * \
            (np.array(exip)-1j*np.array(exim))  # yes,-

        amat = amate+amatm+amatp
        enoi1[i] = hermitianize(amat)
    return enoi1


# electron noise generator
def enoise(efric, exim, exip, bias, T, ecut, dt, nmd, classical=False, zpmotion=True):
    """
    NO SHAPE CHECKING HERE

    shape of efric,exim,exip (md.nph,md.nph)

    efric       electronic friction coefficient
    exim,exip
    bias        
    T           temperature
    ecut        cutoff energy
    dt          md timestep
    nmd         total md steps
    """
    nx = np.shape(np.array(efric))[0]

    hlen = int(nmd/2)
    dw = 2.0*np.pi/dt/nmd
    delta = dt*nmd  # Dirac delta delta

    enoi1 = []
    print("Progress of electron noise generate")
    for i in tqdm(range(hlen+1), unit="steps", mininterval=1):
        w = dw*i
        # equilibrium part
        aw = delta*equ(w, ecut, T, classical, zpmotion)
        amate = aw*np.array(efric)
        # nonequilibrium minus part
        awm = delta*equ(U.hbar*w-bias, ecut, T, classical, zpmotion)
        amatm = -0.5*aw*np.array(exip)+0.5*awm * \
            (np.array(exip)+1j*np.array(exim))
        # nonequilibrium minus part
        awp = delta*equ(U.hbar*w+bias, ecut, T, classical, zpmotion)
        amatp = -0.5*aw*np.array(exip)+0.5*awp * \
            (np.array(exip)-1j*np.array(exim))

        amat = amate+amatm+amatp
        amath = hermitianize(amat)
        # eigh only takes half of the matrix, always return real eigenvalues and
        # vectors
        av, au = LA.eigh(amath)
        # generate multivariance gaussian random numbers
        enoi1.append(vargau(av, au))
    enoi1 = np.array(enoi1)

    enoi2 = []
    for i in range(hlen):
        enoi2.append(np.conjugate(enoi1[hlen-i, :]))
    enoi2 = np.array(enoi2)

    enoi1 = np.delete(enoi1, -1, 0)  # delete last row
    enoi = np.concatenate((enoi1, enoi2), axis=0)

    enoit = []
    fti = myfft(dt, nmd)
    for i in range(nx):
        enoit.append(fti.iFourier1D(enoi[:, i]))  # w->t
    return np.transpose(np.array(enoit))


# --------------------------------------------------------------------------------------
# driver routine for generating noise
def nonequm(w, bias, T, classical=False):
    """
    w       frequency
    bias    applied bias
    T       equilibrium electronic temperature
    """
    hw1 = U.hbar*w-bias
    hw2 = U.hbar*w
    small = 10e-20
    if classical:
        if hw1 == 0.:
            hw1 = small
        if hw2 == 0.:
            hw2 = small
        return 2.0*hw1*(U.kb*T/hw1-U.kb*T/hw2)
    else:
        return 2.0*hw1*(bose(hw1, T)-bose(hw2, T))


def nonequp(w, bias, T, classical=False):
    """
    w       frequency
    bias    applied bias
    T       equilibrium electronic temperature
    """
    hw1 = U.hbar*w+bias
    hw2 = U.hbar*w
    small = 10e-20
    if classical:
        if hw1 == 0.:
            hw1 = small
        if hw2 == 0.:
            hw2 = small
        return 2.0*hw1*(U.kb*T/hw1-U.kb*T/hw2)
    else:
        return 2.0*hw1*(bose(hw1, T)-bose(hw2, T))


def equ(w, cut, T, classical=False, zpmotion=True):
    """
    w       frequency
    cut     electron band cutoff energy
    T       equilibrium electronic temperature
    """
    # small=10e-20
    hw = U.hbar*w
    if zpmotion is True:
        zp = 0.5
    else:
        zp = 0.0
    if(hw < cut):
        if classical:
            return 2.0*U.kb*T
        else:
            if hw == 0:
                return 2.0*U.kb*T
            else:
                return 2.0*hw*(zp+bose(hw, T))
    else:
        return 0.0


def vargau(eval, evec, cof=1.0):
    """
    generate multivariant gaussian:
    eval        a real vector made from the covariance of each dimension
    evec        each COLUMN of evec is one eigen vector 
                NOTE THE DIFFERENCE WITH THE MATHEMATICA CODE
    cof         real coefficient

    All of them should be real.
    Not checked here.
    """
    # print "vargau: checking eigen values and vectors"
    lval = len(eval)
    #nval = np.array(eval)
    nvec = np.array(evec)
    svec = np.shape(nvec)
    if(lval != svec[0] or svec[0] != svec[1]):
        print("vargau: shape error")
        sys.exit(0)
    # if(min(nval) < -10**-10):
    # print "vargau: WARNING, found negative eigenvalue"
    # print "vargau: minimum negative value: ", min(nval)
    # print nval
    # stop
    sval = [cof*v for v in eval]
    rval = []
    for i in range(len(sval)):
        if(sval[i] <= 0):
            rval.append(0.0)
        else:
            rval.append(np.random.normal(0.0, np.sqrt(sval[i])))
    tmm = mdot(nvec, rval)
    return tmm



#!/usr/bin/env python
# -*- coding: utf-8 -*

import numpy as np


def dumpdisp(lammpsdata, trajectoriesfiles, index=[1], outputname="dispstructure"):
    '''
    dump the index.th max displacement from MD trajectories files
    '''
    def sumdisp(inputlist):
        return sum([_ ** 2 for _ in inputlist])

    def giveindex(matrix, index):
        needindex = []
        for i in index:
            disp = [sumdisp(_) for _ in matrix]
            # needvaule = np.sort(disp)[-index]
            needindex.append(np.argsort(disp)[-i])
        return needindex
    from ovito.io import export_file, import_file
    print("import LAMMPS data file %s" % lammpsdata)
    data = import_file(lammpsdata).source.compute(0)
    intposition = np.array(data.particles.positions).flatten()
    positionmatrix = []
    for trajfile in trajectoriesfiles:
        print("import trajectorie file %s" % trajfile)
        traj = import_file(trajfile, columns=[
            "Particle Type", "Position.X", "Position.Y", "Position.Z"])
        for frame_index in range(traj.source.num_frames):
            positionmatrix.append(np.array(traj.source.compute(
                frame_index).particles.positions).flatten())
    #print(giveindex(positionmatrix-intposition, index))
    for i, val in enumerate(giveindex(positionmatrix-intposition, index)):
        data.particles_.positions_[:] = positionmatrix[val].reshape(
            int(len(positionmatrix[val])/3), 3)
        print("export LAMMPS data file %d" % index[i])
        export_file(data, outputname+"." +
                    str(index[i])+".data", "lammps/data", atom_style="full")


def dumpavetraj(lammpsdata, trajectoriesfiles, position_only=False, outputname="avestructure.data"):
    from ovito.io import export_file, import_file

    # export average atom postions from MD trajectories files
    print("import LAMMPS data file %s" % lammpsdata)
    data = import_file(lammpsdata).source.compute(0)
    aveposition = np.zeros(
        [len(trajectoriesfiles), data.number_of_particles, 3])
    if position_only:
        for i, trajfile in enumerate(trajectoriesfiles):
            print("import trajectorie file %s" % trajfile)
            traj = import_file(trajfile, columns=[
                "Particle Type", "Position.X", "Position.Y", "Position.Z"])
            for frame_index in range(traj.source.num_frames):
                position = np.array(traj.source.compute(
                    frame_index).particles.positions)
                aveposition[i] = (aveposition[i]*frame_index +
                                  position)/(frame_index+1)
    else:
        for i, trajfile in enumerate(trajectoriesfiles):
            print("import trajectorie file %s" % trajfile)
            traj = import_file(trajfile, columns=[
                "Particle Type", "Position.X", "Position.Y", "Position.Z", "Force.X", "Force.Y", "Force.Z"])
            for frame_index in range(traj.source.num_frames):
                position = np.array(traj.source.compute(
                    frame_index).particles.positions)
                aveposition[i] = (aveposition[i]*frame_index +
                                  position)/(frame_index+1)
    data.particles_.positions_[:] = np.mean(aveposition, axis=0)
    print("export LAMMPS data file %s" % outputname)
    export_file(data, outputname, "lammps/data", atom_style="full")


def calHF(dlist=1, bathnum=2):
    import glob

    # calculate average heat flux
    print("Calculate heat flux.")
    # temperture=temp
    for filename in glob.glob('kappa.*.bath0.run0.dat'):
        with open(filename, 'r') as f:
            for line in f:
                temperture = float(line.split()[1])

    dlist = list(range(dlist))
    times = int(len(glob.glob('kappa.*.bath0.run*.dat')))
    kb = np.empty([bathnum, times])

    for i in range(bathnum):
        for j in range(times):
            kappafile = "kappa." + \
                str(int(temperture))+".bath"+str(i)+".run"+str(j)+".dat"
            for files in glob.glob(kappafile):
                with open(files, 'r') as f:
                    for line in f:
                        kb[i][j] = line.split()[2]
#                        temperture=float(line.split()[1])
    oldkb = np.delete(kb, dlist, axis=1)
    balancekb = np.delete(kb, dlist, axis=1)
    for i in range(balancekb.shape[0]):
        for j in range(balancekb.shape[1]):
            balancekb[i][j] = np.mean(oldkb[i][0:j+1])

    np.savetxt('heatflux.'+str(int(temperture)) +
               '.dat', np.transpose(balancekb))


def calTC(delta, dlist=1, L=None, A=None):
    # L，A units in Angstrom and Angstrom**2, respectively
    import glob

    # calculate thermal conductance
    print("Calculate thermal conductance.")
    delta = delta
    # temperture=temp
    for filename in glob.glob('kappa.*.bath0.run0.dat'):
        with open(filename, 'r') as f:
            for line in f:
                temperture = float(line.split()[1])
    dlist = list(range(dlist))
    times = int(len(glob.glob('kappa.*.bath*.run*.dat'))/2)
    kb = np.empty([2, times])

    for i in range(2):
        for j in range(times):
            kappafile = "kappa." + \
                str(int(temperture))+".bath"+str(i)+".run"+str(j)+".dat"
            for files in glob.glob(kappafile):
                with open(files, 'r') as f:
                    for line in f:
                        kb[i][j] = line.split()[2]
#                        temperture=float(line.split()[1])
    if delta != 0:
        kappa = (kb[0]-kb[1])/2/(delta*temperture)
        kappa = np.delete(kappa, dlist)
        # for i in range(len(kappa)):
        #    kappa[i]=np.mean(kappa[0:i+1])

        np.savetxt('thermalconductance.'+str(int(temperture))+'.dat',
                   (np.mean(kappa), np.std(kappa)), header="Mean(nW/K) Std(nW/K)")
        if L is not None and A is not None:
            np.savetxt('thermalconductivity.'+str(int(temperture))+'.dat',
                       (np.mean(kappa*L/A*10), np.std(kappa*L/A*10)), header="Mean(W/m-K) Std(W/m-K)")
    else:
        print("delta=0, no thermal conductance/conductivity calculated.")


def get_atomname(mass):
    """
    get the element name from its atomic mass by checking the dictionary
    """
    import sclmd.units as U

    for key, value in list(U.AtomicMassTable.items()):
        if abs(mass-value) < 0.01:
            return key


def eff():
    '''
    eliminate false frequencies
    '''
    dynmatdat = np.loadtxt('dynmat.dat')
    dynlen = int(3*np.sqrt(len(dynmatdat)/3))
    dynmat = dynmatdat.reshape((dynlen, dynlen))
    #dynmat = (dynmat+dynmat.conjugate().transpose())/2
    eigvals, eigvecs = np.linalg.eigh(dynmat)
    while not (eigvals > 0).all():
        for i, val in enumerate(eigvals):
            if val < 0:
                print('False frequency exists in system DOF %i ' % i)
                eigvals[i] = 0
        dynmat = np.linalg.multi_dot([eigvecs, np.identity(
            len(eigvals))*eigvals, np.linalg.inv(eigvecs)])
        eigvals, eigvecs = np.linalg.eigh(dynmat)
    np.savetxt('dynmatmod.dat', dynmat)


if __name__ == "__main__":
    from sclmd.tools import dumpavetraj
    lammps = "structure.data"
    trajectories = ["trajectories.300.run0.ani", "trajectories.300.run1.ani",
                    "trajectories.300.run2.ani", "trajectories.300.run3.ani", "trajectories.300.run4.ani"]
    dumpavetraj(lammps, trajectories, position_only=False,
                outputname="avetrajectories.data")
    avefiles = ["avestructure.300.run0.dat", "avestructure.300.run1.dat",
                "avestructure.300.run2.dat", "avestructure.300.run3.dat", "avestructure.300.run4.dat"]
    dumpavetraj(lammps, avefiles, position_only=True,
                outputname="avestructure.data")

    from sclmd.tools import dumpdisp
    lammps = "avestructure.data"
    trajectories = ["trajectories.100.run0.ani", "trajectories.100.run1.ani",
                    "trajectories.100.run2.ani", "trajectories.100.run3.ani",
                    "trajectories.100.run4.ani", "trajectories.100.run5.ani",
                    "trajectories.100.run6.ani", "trajectories.100.run7.ani",
                    "trajectories.100.run8.ani", "trajectories.100.run9.ani",
                    "trajectories.100.run10.ani", "trajectories.100.run11.ani",
                    "trajectories.100.run12.ani", "trajectories.100.run13.ani",
                    "trajectories.100.run14.ani", "trajectories.100.run15.ani",
                    "trajectories.100.run16.ani", "trajectories.100.run17.ani",
                    "trajectories.100.run18.ani", "trajectories.100.run19.ani",
                    ]
    dumpdisp(lammps, trajectories, index=[
             1, 10, 100], outputname="dispstructure")



#!/usr/bin/env python
# -*- coding: utf-8 -*

import sys
import numpy as np
from lammps import lammps


class bpt:
    # Use NEGF to calculate ballistic phonon transport
    def __init__(self, infile, maxomega, damp, dofatomofbath, dofatomfixed=[[], []], dynmatfile=None, num=1000, vector=False):
        print('Class init')
        # reduced Planck constant unit in: eV*ps
        self.rpc = 6.582119569e-4
        # Boltzmann constant unit in: eV/K
        self.bc = 8.617333262e-5
        self.damp = damp
        self.maxomega = maxomega/self.rpc
        self.intnum = num
        self.dofatomfixed = dofatomfixed
        self.dofatomofbath = dofatomofbath
        self.dynmatfile = dynmatfile
        self.getdynmat(infile)
        self.gettm(vector)

    def getdynmat(self, infile):
        lmp = lammps()
        #lmp = lammps(cmdargs=['-screen', 'none', '-log', 'none'])
        print('LAMMPS init')
        lmp.commands_list(infile)
        self.natoms = lmp.get_natoms()
        box = lmp.extract_box()
        self.boxlo = np.array(box[0])
        self.boxhi = np.array(box[1])
        systype = np.array(lmp.gather_atoms("type", 0, 1))
        mass = lmp.extract_atom("mass", 2)
        self.els = []
        for type in systype:
            self.els.append([mass[type]]*3)
        self.els = np.array(self.els).flatten()
        self.xyz = lmp.gather_atoms("x", 1, 3)
        self.els = np.delete(self.els, self.dofatomfixed[0])
        self.els = np.delete(self.els, [
            dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]])
        self.xyz = np.delete(self.xyz, self.dofatomfixed[0])
        self.xyz = np.delete(self.xyz, [
            dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]])
        if self.dynmatfile is None:
            print('Calculate dynamical matrix')
            lmp.command('dynamical_matrix all eskm 0.000001 file dynmat.dat')
            dynmatdat = np.loadtxt('dynmat.dat')
        else:
            print('Load dynamical matrix from '+str(self.dynmatfile))
            dynmatdat = np.loadtxt(self.dynmatfile)
        lmp.close()
        self.dynmat = []
        self.omegas = []
        self.doffreeatom = 0
        dynlen = int(3*np.sqrt(len(dynmatdat)/3))
        if dynlen != self.natoms*3:
            print('System DOF test failed after load dynmat, check again')
            sys.exit()
        self.dynmat = dynmatdat.reshape((dynlen, dynlen))
        self.dynmat = np.delete(self.dynmat, self.dofatomfixed[0], axis=0)
        self.dynmat = np.delete(self.dynmat, self.dofatomfixed[0], axis=1)
        self.dynmat = np.delete(self.dynmat, [
                                dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]], axis=0)
        self.dynmat = np.delete(self.dynmat, [
                                dof-len(self.dofatomfixed[0]) for dof in self.dofatomfixed[1]], axis=1)
        if len(self.xyz) != len(self.dynmat):
            print('System DOF test failed after atoms reduced, check again')
            sys.exit()
        ffi = []
        print('Calculate angular frequency')
        eigvals, self.eigvecs = np.linalg.eigh(self.dynmat)
        for i, val in enumerate(eigvals):
            if val > 0:
                self.omegas.append(np.sqrt(val)*self.rpc)
            else:
                ffi.append(i)
                # print('False frequency exists in system DOF %i ' %
                #      (i+len(self.dofatomfixed[0])))
                self.omegas.append(-np.sqrt(-val)*self.rpc)
        print('%i false frequencies exist in %i frequencies' %
              (len(ffi), len(self.omegas)))
        np.savetxt('falsefrequencies.dat', ffi, fmt='%d')
        np.savetxt('omegas.dat', self.omegas)
        np.savetxt('eigvecs.dat', self.eigvecs)

    def gettm(self, vector):
        print('Calculate transmission')
        x = np.linspace(0, self.maxomega, self.intnum+1)
        if vector:
            function = np.vectorize(self.tm)
            self.tmnumber = np.array(
                np.column_stack((x, np.array(function(x)))))
        else:
            from tqdm import tqdm
            tm = []
            for var in tqdm(x, unit="steps", mininterval=1):
                tm.append(self.tm(var))
            self.tmnumber = np.array(np.column_stack((x, np.array(tm))))
        np.savetxt('transmission.dat', np.column_stack(
            (self.tmnumber[:, 0]*self.rpc, self.tmnumber[:, 1])))
        print('Transmission saved')

    def getps(self, T, maxomega, intnum, atomlist=None, filename=None, vector=False):
        if filename is not None:
            print('Calculate power spectrum at '+str(T)+'K of'+str(filename))
        else:
            print('Calculate power spectrum at '+str(T)+'K')
        if atomlist is None:
            print("Power spectrum of all atoms")
            atomlist = np.array(range(0, len(self.dynmat))
                                ) + len(self.dofatomfixed[0])
        x2 = np.linspace(0, maxomega/self.rpc, intnum+1)
        if vector:
            function = np.vectorize(self.ps)
            self.psnumber = np.array(
                np.column_stack((x2, np.array(function(x2, T, atomlist)))))
        else:
            from tqdm import tqdm
            ps = []
            for var in tqdm(x2, unit="steps", mininterval=1):
                ps.append(self.ps(var, T, atomlist))
            self.psnumber = np.array(np.column_stack((x2, np.array(ps))))
        if filename is not None:
            np.savetxt('powerspectrum.'+str(filename)+'.'+str(T)+'.dat',
                       np.column_stack((self.psnumber[:, 0]*self.rpc, self.psnumber[:, 1])))
        else:
            np.savetxt('powerspectrum.'+str(T)+'.dat',
                       np.column_stack((self.psnumber[:, 0]*self.rpc, self.psnumber[:, 1])))
        print('Power spectrum saved')

    def selfenergy(self, omega, dofatoms):
        return -1j*omega*(1/self.damp)*self.atomofbath(dofatoms)

    def atomofbath(self, dofatoms):
        semat = np.zeros((self.natoms*3, self.natoms*3))
        for dofatom in dofatoms:
            semat[dofatom][dofatom] = 1
        semat = np.delete(semat, self.dofatomfixed[0], axis=0)
        semat = np.delete(semat, self.dofatomfixed[0], axis=1)
        semat = np.delete(semat, [dof-len(self.dofatomfixed[0])
                                  for dof in self.dofatomfixed[1]], axis=0)
        semat = np.delete(semat, [dof-len(self.dofatomfixed[0])
                                  for dof in self.dofatomfixed[1]], axis=1)
        if len(semat) != len(self.dynmat) or self.natoms*3 != len(self.dofatomfixed[0]) + len(self.dofatomfixed[1]) + len(semat):
            print('System DOF test failed, check again')
            sys.exit()
        return semat

    def retargf(self, omega):
        # retarded Green function
        return np.linalg.inv((omega+1e-9j)**2*np.identity(len(self.dynmat))-self.dynmat-self.selfenergy(omega, self.dofatomofbath[0])-self.selfenergy(omega, self.dofatomofbath[1]))

    def gamma(self, Pi):
        return -1j*(Pi-Pi.conjugate().transpose())

    def bosedist(self, omega, T):
        # Bose Einstein distribution
        if abs(T) < 1e-30:
            # print('T %e is too small. Set kBT min.' % T)
            return 1/(np.exp(self.rpc*omega*np.iinfo(np.int32).max)-1)
        elif abs(omega/T) < 1e-30:
            # print('omega %e is too small. Set bose einstein distribution max.' % omega)
            return np.iinfo(np.int32).max
        else:
            return 1/(np.exp(self.rpc*omega/self.bc/T)-1)

    def ps(self, omega, T, atomlist):
        # Power spectrum of selected atoms
        dofatomse = np.array(atomlist)-len(self.dofatomfixed[0])
        return -2*omega**2*self.bosedist(omega, T)*np.trace(np.imag(self.retargf(omega)[dofatomse][:, dofatomse]))

    def tm(self, omega):
        # Transmission
        return np.real(np.trace(np.dot(np.dot(np.dot(self.retargf(omega), self.gamma(self.selfenergy(
            omega, self.dofatomofbath[0]))), self.retargf(omega).conjugate().transpose()), self.gamma(self.selfenergy(omega, self.dofatomofbath[1])))))

    def thermalcurrent(self, T, delta):
        # def f(omega):
        #    return self.rpc*omega/2 / \
        #        np.pi*self.tm(omega)*(self.bosedist(omega, T*(1+0.5*delta)) -
        #                              self.bosedist(omega, T*(1-0.5*delta)))

        # def trape(function, maxnumber, n):
        #    function = np.vectorize(function)
        #    arr = function(np.linspace(0, maxnumber, n+1))
        #    return (float(maxnumber - 0)/n/2.)*(2*arr.sum() - arr[0] - arr[-1])

        def f(i):
            return self.rpc*self.tmnumber[i, 0]/2 / \
                np.pi*self.tmnumber[i, 1]*(self.bosedist(self.tmnumber[i, 0], T*(1+0.5*delta)) -
                                           self.bosedist(self.tmnumber[i, 0], T*(1-0.5*delta)))

        def trape(function):
            n = len(self.tmnumber[:, 0]) - 1
            if n != self.intnum:
                print('Error in number of omega')
                sys.exit()
            function = np.vectorize(function)
            arr = function(range(n+1))
            return (float(self.tmnumber[-1, 0] - self.tmnumber[0, 0])/n/2.)*(2*arr.sum() - arr[0] - arr[-1])
        # Unit in nW
        # return trape(f, self.maxomega, self.intnum)*1.60217662*1e2
        return trape(f)*1.60217662*1e2

    def thermalconductance(self, T, delta):
        return self.thermalcurrent(T, delta)/(T*delta)

    def thermalconductivity(self, T, delta, L, A):
        # L,A units in Angstrom
        return self.thermalconductance(T, delta)*L/A*10

    def write_v_sim(self, filename="anime.ascii"):
        from sclmd.tools import get_atomname
        # TODO: Not completely accurate in box setting & eigvecs
        text = "# Generated file for v_sim 3.7\n"
        text += "%15.9f%15.9f%15.9f\n" % (
            self.boxhi[0], self.boxlo[2], self.boxhi[1])
        text += "%15.9f%15.9f%15.9f\n" % (
            self.boxlo[0], self.boxlo[1], self.boxhi[2])
        for i in range(int(len(self.els)/3)):
            text += "%15.9f%15.9f%15.9f %2s\n" % (
                self.xyz[3*i], self.xyz[3*i+1], self.xyz[3*i+2], get_atomname(self.els[3*i]))
        for i, a in enumerate(self.omegas):
            text += "#metaData: qpt=[%f;%f;%f;%f \\\n" % (0, 0, 0, a)
            for u in range(int(len(self.els)/3)):
                text += "#; %f; %f; %f; %f; %f; %f \\\n" % (
                    self.eigvecs[i, 3*u]/self.els[3*u]**0.5, self.eigvecs[i, 3*u+1]/self.els[3*u]**0.5, self.eigvecs[i, 3*u+2]/self.els[3*u]**0.5, 0, 0, 0)
            text += "# ]\n"
        vfile = open(filename, 'w')
        vfile.write(text)
        vfile.close()

    def plotresult(self, lines=180):
        from matplotlib import pyplot as plt
        plt.figure(0)
        plt.hist(self.omegas, bins=lines)
        plt.xlabel('Frequence(eV)')
        plt.ylabel('Number')
        #plt.xlim(0, self.maxomega*self.rpc)
        plt.savefig('omegas.png')
        plt.figure(1)
        plt.plot(self.tmnumber[:, 0]*self.rpc, self.tmnumber[:, 1])
        plt.xlabel('Frequence(eV)')
        plt.ylabel('Transmission')
        plt.savefig('transmission.png')


if __name__ == '__main__':
    '''
    Units
    Time: ps
    Frequence: eV
    Temperture: K
    Heat Current: nW
    '''
    import time
    import numpy as np
    from sclmd.negf import bpt
    from matplotlib import pyplot as plt
    infile = ['atom_style full',
              'units metal',
              'boundary f p p',
              'read_data structure.data',
              'pair_style rebo',
              'pair_coeff * * CH.rebo C H',
              ]
    time_start = time.time()
    atomfixed = [range(0*3, (19+1)*3), range(181*3, (200+1)*3)]
    atomofbath = [range(20*3, (69+1)*3), range(131*3, (180+1)*3)]
    mybpt = bpt(infile, 0.25, 0.1, atomofbath, atomfixed, 100)
    mybpt.plotresult()
    # T_H/C = T*(1±delta/2)
    T = [100, 200, 300, 400, 500, 600, 700,
         800, 900, 1000]
    delta = 0.1
    thermalconductance = []
    for temp in T:
        thermalconductance.append(
            [temp, mybpt.thermalconductance(temp, delta)])
    mybpt.getps(300, 0.5, 1000)
    np.savetxt('thermalconductance.dat', thermalconductance)
    plt.figure(5)
    plt.plot(np.array(thermalconductance)[
        :, 0], np.array(thermalconductance)[:, 1])
    plt.xlabel('Temperature(K)')
    plt.ylabel('Thermal Conductance(nW/K)')
    plt.savefig('thermalconductance.png')
    time_end = time.time()
    print('time cost', time_end-time_start, 's')
